import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.Comparator;
import java.util.ListIterator;
import java.util.Random;
import java.util.Vector;

/*
 * Author: Jason Bittel
 * Date:   May 26, 2004
 * 
 * Description:
 * This program generates a random connected graph and then
 * utilizes Kruskal's algorithm to compute the minimum spanning 
 * tree for the graph.
 */

public class MinSpanTree {
	public static final int MAXEDGEWEIGHT = 15;
	
	static Vector allEdges = new Vector();
	static Vector allVertices = new Vector();
	static Vector connVertices = new Vector();
	static Vector minEdges = new Vector();

	public static void main(String[] args) {
		int numVertices = 0;
		int numEdges = 0;
		
		// Gather command line parameters
		if(args.length == 2) {
			numVertices = Integer.parseInt(args[0]);
			numEdges = Integer.parseInt(args[1]);
		} else {
			System.out.println("Usage: MinSpanTree [number of vertices] [number of edges]");
			return;
		}
		
		// Generate a random connected graph
		generateGraph(numVertices, numEdges);

		// Compute the minimum spanning tree
		mstKruskal();
		
		// Print the results of the algorithm
		printGraph(numVertices, numEdges);
	}
	
	private static void mstKruskal() {
		minEdges.clear();
		
		// Make singleton set for each vertex
		ListIterator vertices = allVertices.listIterator();
		while(vertices.hasNext()) {
			Vertex nextVertex = (Vertex) vertices.next();
			nextVertex.addSetElement(nextVertex);
		}
		
		// Sort edges of E as nondecreasing
		Collections.sort(allEdges, new Comparator()	{
				public int compare(Object firstEdge, Object secondEdge)	{
					return ((Edge) firstEdge).getWeight() - ((Edge) secondEdge).getWeight();
				}
			}	
		);
		
		// For each edge (u,v) in E
		ListIterator edges = allEdges.listIterator();
		while(edges.hasNext()) {
			Edge nextEdge = (Edge) edges.next();
			// If Su is not in Sv
			if(!nextEdge.getV().setContains(nextEdge.getU())) {
				// Replace Su and Sv with Su U Sv
				nextEdge.getV().addSetElement(nextEdge.getU().set);
				nextEdge.getV().unionSet();				
				// Add (u,v) to set of MST
				minEdges.add(nextEdge);
			}
		}
	}
	
	private static void generateGraph(int numVertices, int numEdges) {
		Random randNum = new Random();
		
		// Populate vertices
		for (int i = 0; i < numVertices; i++) {
			allVertices.add(new Vertex(i));
		}
		
		// Populate edges
		for (int i = 0; i < numEdges; i++) {
			Vertex vertexU = null;
			Vertex vertexV = null;
			
			// Prevent self loops
			while(vertexU == vertexV) {
				// Ensure connected graph
				if(i == 0) {
					vertexU = (Vertex)allVertices.get(randNum.nextInt(allVertices.size()));
					connVertices.add(vertexU);
				} else {
					vertexU = (Vertex)connVertices.get(randNum.nextInt(connVertices.size()));
				}

				vertexV = (Vertex)allVertices.get(randNum.nextInt(allVertices.size()));
			}
			
			if(!connVertices.contains(vertexV)) {
				connVertices.add(vertexV);
			}
			
			allEdges.add(new Edge(i, vertexU, vertexV, randNum.nextInt(MAXEDGEWEIGHT)));
		}
	}

	private static void printGraph(int numVertices, int numEdges) {
		PrintWriter outFile = null;
		String fileName = numVertices + "x" + numEdges + ".txt";
		
		try {
			outFile = new PrintWriter(new FileWriter(fileName));
			outFile.println("digraph G {");
			outFile.println("\tgraph [overlap=scale];");
			outFile.println("\tnode [shape=circle, style=filled, fontsize=12, height=.25];");
			outFile.println("\tedge [fontsize=12, arrowhead=none, arrowtail=none];");
			// Print all edges
			ListIterator edges = allEdges.listIterator();
			while(edges.hasNext()) {
				Edge currEdge = (Edge) edges.next();
				if(!minEdges.contains(currEdge)) {
					outFile.println("\t" + currEdge.getU().getID() + " -> " + currEdge.getV().getID() + " [label=\"" + currEdge.getWeight() + "\"];");
				}
			}
			// Print MST
			outFile.println("\tedge [fontsize=12, style=bold, arrowhead=none, arrowtail=none];");
			ListIterator mst = minEdges.listIterator();
			while(mst.hasNext()) {
				Edge currEdge = (Edge) mst.next();
				outFile.println("\t" + currEdge.getU().getID() + " -> " + currEdge.getV().getID() + " [label=\"" + currEdge.getWeight() + "\", weight=5];");
			}
			outFile.println("}");	
			outFile.close();
		} catch (IOException e) {
			System.out.println(e);
		}		
	}
}
